using System;
using System.Collections.Generic;
using System.Text;
using Iesi.Collections;
using System.Collections;

namespace Association_mining
{
    /// <summary>
    /// A static class containing methods used to mine frequent sets from a database.
    /// </summary>
    public static class Apriori
    {
        /// <summary>
        /// Generates a list of frequent sets form a  database.
        /// </summary>
        /// <param name="data">Database to generate frequent sets from</param>
        /// <param name="minsup">Desired minimal support</param>
        /// <returns>A list of frequent sets fulfilling the minimal support
        /// requirement.</returns>
        public static List<FrequentSet> GenerateFrequentSets(Database data, double minsup)
        {
            List<Set> ItemList = new List<Set>();
            List<Set> CandidateList = new List<Set>();
            List<FrequentSet> FrequentItems = new List<FrequentSet>();

            int UniverseCount = data.RowCount;
            double MinimalSupport = (double)(minsup * UniverseCount);

            for (int i = 0; i < data.Template.NameCount; i++)
            {
                if (data.Template.AttributeSupport(i) >= MinimalSupport)
                    ItemList.Add(new HashedSet(new Item[] { new Item(data.Template.Types[i], data.Template.Names[i]) }));
            }
            CandidateList = SetUtil.GenerateCandidateSets(ItemList);

            //iteracyjne wyszukiwanie zbiorow czestych
            while (CandidateList.Count > 0)
            {
                Dictionary<int,long> SupportCount = new Dictionary<int,long>();
                for (int i = 0; i < CandidateList.Count; i++ )
                    SupportCount.Add(i, 0);

                //we run through the database to check candidate sets support
                for (int i = 0; i < UniverseCount; i++)
                {
                    for (int j = 0; j < CandidateList.Count; j++)
                    {
                        Set s = CandidateList[j];
                        bool sup = true;
                        foreach (Item x in s)
                        {
                            if (data[i, data.Translator[x.Type+x.Name]] == 0)
                                sup = false;
                        }
                        if (sup)
                            SupportCount[j]++;
                    }
                }

                List<Set> RejectList = new List<Set>();

                foreach (KeyValuePair<int, long> kvp in SupportCount)
                {
                    if (kvp.Value >= MinimalSupport)
                        FrequentItems.Add(new FrequentSet(CandidateList[kvp.Key], kvp.Value, UniverseCount));
                    else
                        RejectList.Add(CandidateList[kvp.Key]);
                }
                foreach (Set s in RejectList)
                    CandidateList.Remove(s);

                CandidateList = SetUtil.GenerateCandidateSets(CandidateList);
            }
            return FrequentItems;
        }

        /// <summary>
        /// Implementation of the Apriori algorithm.
        /// </summary>
        /// <param name="MainForm">The main program form</param>
        /// <param name="minsup">Minimal support</param>
        /// <param name="data">Database to generate rules from</param>
        /// <param name="ConclusionList">A list of conclusions</param>
        /// <param name="maxanti">Maximal anti-support</param>
        /// <param name="minconf">Minimal confidence</param>
        /// <param name="minf">Minimal confirmation-f</param>
        /// <param name="mins">Minimal confirmation-s</param>
        /// <returns>A list of rules matching the minimal support 
        /// and other requirements</returns>
        public static SortableBindingList<Rule> MiningAlgorithm(frmMain MainForm, Database data, List<Set> ConclusionList, 
            double minsup, double minconf, double mins, double minf, double maxanti, string decClass)
        {
            List<FrequentSet> fsList = new List<FrequentSet>();
            // First generate a list frequent sets
            fsList = GenerateFrequentSets(data, minsup);
            if (decClass == "")
            {
                // Then generate rules from that list
                return SetUtil.GenerateRules(MainForm, data, ConclusionList, fsList, minconf, mins, minf, maxanti);
            }
            else
            {
                // Then generate rules from that list
                return SetUtil.GenerateRules(MainForm, data, ConclusionList, fsList, minconf, mins, minf, maxanti, decClass);
            }
            
        }
    }
}
