using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Iesi.Collections;

namespace Association_mining
{
    /// <summary>
    /// Class providing an interface for the chart creation process.
    /// </summary>
    public partial class CreateChart : Form
    {
        /// <summary>
        /// A list of rules with the same conclusion.
        /// </summary>
        protected List<Rule> FilteredList;
        /// <summary>
        /// Rule structure.
        /// </summary>
        protected List<List<Rule>> Rules;
        /// <summary>
        /// Minimal support used to generate the rules in the structure.
        /// </summary>
        protected double MinSup, MinConf, MinS, MinF, MaxAnti;

        /// <summary>
        /// Constructor.
        /// <param name="ruleList">Rule structure</param>
        /// <param name="Conclusions">List of conclusions</param>
        /// <param name="minsup">Minimal support</param>
        /// <param name="minconf">Minimal confidence</param>
        /// <param name="mins">Minimal confirmation-s</param>
        /// <param name="minf">Minimal confirmation-f</param>
        /// <param name="maxa">Maximal anti-support</param>
        /// </summary>
        public CreateChart(List<List<Rule>> ruleList, List<Set> Conclusions, 
            double minsup, double minconf, double mins, double minf, double maxa)
        {
            int maxWidth = 0;
            string hipothesis="";

            MinSup = minsup;
            MinConf = minconf;
            MinS = mins;
            MinF = minf;
            MaxAnti = maxa;

            Rules = ruleList;
            FilteredList = new List<Rule>();
            InitializeComponent();

            foreach (Set s in Conclusions)
            {
                comboBox1.Items.Add(hipothesis = SetUtil.ParseSet(s));
                if (hipothesis.Length > maxWidth)
                    maxWidth = hipothesis.Length;
            }

            comboBox1.DropDownWidth = maxWidth * 5 + 2;
            comboBox1.SelectedIndex = 0;
         
        }

        /// <summary>
        /// Destroys the form.
        /// </summary>
        protected void Cancel_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        /// <summary>
        /// Creates the specified chart.
        /// </summary>
        protected void OK_Click(object sender, EventArgs e)
        {
            FilteredList = Rules[comboBox1.SelectedIndex];
            if (cboxConf.Checked)
            {
                Chart c = new Chart(FilteredList, 1, MinSup, MinConf, MinS, MinF, MaxAnti);
                c.Show();
                c.tboxSup.Focus();
            }
            if (cboxS.Checked)
            {
                Chart c = new Chart(FilteredList, 2, MinSup, MinConf, MinS, MinF, MaxAnti);
                c.Show();
                c.tboxSup.Focus();
            }
            if (cboxF.Checked)
            {
                Chart c = new Chart(FilteredList, 3, MinSup, MinConf, MinS, MinF, MaxAnti);
                c.Show();
                c.tboxSup.Focus();
            }
            if (cboxASup.Checked)
            {
                Chart c = new Chart(FilteredList, 4, MinSup, MinConf, MinS, MinF, MaxAnti);
                c.Show();
                c.tboxSup.Focus();
            }
            if(cboxConf.Checked || cboxS.Checked || cboxF.Checked || cboxASup.Checked)
                Dispose();
            else
                MessageBox.Show("Please select one of the chart types or press Cancel.", "No chart type selected!", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// Selects or deselects all checkboxes at once.
        /// </summary>
        protected void cboxAll_CheckedChanged(object sender, EventArgs e)
        {
            if (cboxAll.Checked)
            {
                cboxConf.Checked = true;
                cboxF.Checked = true;
                cboxS.Checked = true;
                cboxASup.Checked = true;
            }
            else
            {
                cboxConf.Checked = false;
                cboxF.Checked = false;
                cboxS.Checked = false;
                cboxASup.Checked = false;
            }
        }
    }
}