using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Iesi.Collections;

namespace Association_mining
{
    /// <summary>
    /// Class providing an interface for the multichart creation process.
    /// </summary>
    public partial class CreateMultiChart : Form
    {
        /// <summary>
        /// A list of rules with the same conclusion.
        /// </summary>
        protected List<Rule> FilteredList;
        /// <summary>
        /// Rule structure.
        /// </summary>
        protected List<List<Rule>> Rules;
        /// <summary>
        /// Minimal support used to generate the rules in the structure.
        /// </summary>
        protected double MinSup, MinConf, MinS, MinF, MaxAnti;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ruleList">Rule list</param>
        /// <param name="Conclusions">LList of conclusions</param>
        /// <param name="minsup">Minimal support</param>
        /// <param name="minconf">Minimal confidence</param>
        /// <param name="mins">Minimal confirmation-s</param>
        /// <param name="minf">Minimal confirmation-f</param>
        /// <param name="maxa">Maximal anti-support</param>
        public CreateMultiChart(List<List<Rule>> ruleList, List<Set> Conclusions, 
            double minsup, double minconf, double mins, double minf, double maxa)
        {
            int maxWidth = 0;
            string hipothesis="";

            MinSup = minsup;
            MinConf = minconf;
            MinS = mins;
            MinF = minf;
            MaxAnti = maxa;

            Rules = ruleList;
            FilteredList = new List<Rule>();
            InitializeComponent();

            foreach (Set s in Conclusions)
            {
                comboBox1.Items.Add(hipothesis = SetUtil.ParseSet(s));
                if (hipothesis.Length > maxWidth)
                    maxWidth = hipothesis.Length;
            }

            comboBox1.DropDownWidth = maxWidth * 5 + 2;
            comboBox1.SelectedIndex = 0;
         
        }

        /// <summary>
        /// Destroys the form.
        /// </summary>
        protected void Cacel_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        /// <summary>
        /// Creates the specified chart.
        /// </summary>
        protected void OK_Click(object sender, EventArgs e)
        {
            FilteredList = Rules[comboBox1.SelectedIndex];
            MultiChart mc = new MultiChart(FilteredList, 
                                MinSup, MinConf, MinS, MinF, MaxAnti);
            mc.Show();
            mc.tboxSup.Focus();
            Dispose();
        }
    }
}