using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using Iesi.Collections;

namespace Association_mining
{
    public partial class frmMain : Form
    {
        private Database database =  new Database();
        private SortableBindingList<Rule> ruleList;
        private List<Set> ConclusionList = new List<Set>();
        private List<List<Rule>> RuleStructure;
        private double MinSup, MinConf, MinS, MinF, MaxAnti;
        private MinSupp MinSupForm;
        private string decClass;

        //Start and stop times of the rule generation process
        int iStart, iStop;

        private Thread WorkingThread;
        /// <summary>
        /// A delegate used to update rule display.
        /// </summary>
        /// <param name="ruleList">Rule list</param>
        private delegate void DisplayRulesCallback(SortableBindingList<Rule> ruleList);
        /// <summary>
        /// A delegate used invoke functions without parameters by a background thread.
        /// </summary>
        private delegate void ThreadCallback();
        /// <summary>
        /// A delegate used invoke functions with one int parameter by a background thread.
        /// </summary>
        /// <param name="i">A number</param>
        private delegate void IntThreadCallback(int i);

        /// <summary>
        /// Default constructor.
        /// </summary>
        public frmMain()
        {
            InitializeComponent();
            DGV.AutoGenerateColumns = false;
            DGV.Columns[0].DataPropertyName = "premise";
            DGV.Columns[1].DataPropertyName = "conclusion";
            DGV.Columns[2].DataPropertyName = "Support";
            DGV.Columns[3].DataPropertyName = "Confidence";
            DGV.Columns[4].DataPropertyName = "S";
            DGV.Columns[5].DataPropertyName = "F";
            DGV.Columns[6].DataPropertyName = "AntiSupport";
            ruleList = new SortableBindingList<Rule>();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox a = new AboutBox();
            a.Show();
        }

        private void LoadData(object f)
        {
            try
            {
                FileStream file = (FileStream)f;
                string extension = file.Name.Substring(file.Name.Length - 4);
                if (extension == "arff")
                    database = FileProcessor.OpenARFFFile(file);
                else
                    database = FileProcessor.OpenDATAFile(file);
                ShowMinSupDialog();
            }
            catch
            {
                MessageBox.Show("Wrong data file format!", "Data file error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Error();
            }
        }

        private void ShowMinSupDialog()
        {
            if (this.InvokeRequired)
            {
                ThreadCallback t = new ThreadCallback(ShowMinSupDialog);
                this.Invoke(t, new object[] { });
            }
            else
            {
                toolStripStatusLabel1.Text = "Data loaded";
                ProgressBar.Visible = false;
                operationButton.Visible = false;
                shoToolStripMenuItem.Enabled = true;
                changeToolStripMenuItem.Enabled = true;
                MinSupForm = new MinSupp(this, database.Template);
                MinSupForm.Show(this);
                MinSupForm.textBox1.Focus();
            }
        }

        /// <summary>
        /// Generates rules.
        /// </summary>
        public void RuleGenerationThread()
        {
            try
            {
                RuleStructure = new List<List<Rule>>();
                ruleList = new SortableBindingList<Rule>();
                ConclusionList = new List<Set>();

                iStart = Environment.TickCount;
                if (fPGrowthToolStripMenuItem.Checked)
                    ruleList = FPGrowth.MiningAlgorithm(this, database, ConclusionList, MinSup, MinConf, MinS, MinF, MaxAnti, decClass);
                else
                    ruleList = Apriori.MiningAlgorithm(this, database, ConclusionList, MinSup, MinConf, MinS, MinF, MaxAnti, decClass);
                RuleStructure = Rule.Transform(ruleList, ConclusionList);
                iStop = Environment.TickCount;

                CheckRuleListCount();
            }
            catch
            {
                MessageBox.Show("The program has encountered a problem during the rule generation process. Try again with a greater minimal support value.", "Rule generation error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Error();
            }
        }

        /// <summary>
        /// Sets the rule count value on the status strip.
        /// </summary>
        /// <param name="value">Rule count</param>
        public void SetRuleCount(int value)
        {
            if (this.InvokeRequired)
            {
                IntThreadCallback t = new IntThreadCallback(SetRuleCount);
                this.Invoke(t, new object[] { value });
            }
            else
            {
                toolStripRuleCount.Text = value.ToString();
            }
        }

        /// <summary>
        /// Number of generated rules.
        /// </summary>
        public int RuleCount
        {
            get { return Int32.Parse(toolStripRuleCount.Text); }
        }

        private void CheckRuleListCount()
        {
            if (this.InvokeRequired)
            {
                ThreadCallback t = new ThreadCallback(CheckRuleListCount);
                this.Invoke(t, new object[] { });
            }
            else
            {
                ProgressBar.Visible = false;
                if (ruleList.Count == 0)
                {
                    createChartToolStripMenuItem.Enabled = false;
                    createMultichartToolStripMenuItem.Enabled = false;
                    saveRuleFileToolStripMenuItem.Enabled = false;
                    DisplayRules(ruleList);
                    toolStripStatusLabel1.Text = "Done! No rules found.";
                    MessageBox.Show("No rules could be found using the given minimal support!", "No rules found", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    DisplayRules(ruleList);
                    toolStripStatusLabel1.Text = "Done! Rule generation time: "+
                        ((iStop - iStart) / 3600000).ToString() + "h "+
                        ((iStop - iStart) / 60000).ToString() + "min "+
                        ((iStop-iStart)/1000).ToString()+"s.";
                }
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openDataFileDialog.Title = "Choose file...";
            openDataFileDialog.Filter = "Miner File (*.arff;*.data)|*.arff;*.data";
            openDataFileDialog.FilterIndex = 1;
            openDataFileDialog.FileName = "";
            if (openDataFileDialog.ShowDialog() != DialogResult.Cancel)
            {
                shoToolStripMenuItem.Enabled = false;
                createChartToolStripMenuItem.Enabled = false;
                createMultichartToolStripMenuItem.Enabled = false;
                changeToolStripMenuItem.Enabled = false;
                saveRuleFileToolStripMenuItem.Enabled = false;

                FileStream file = new FileStream(openDataFileDialog.FileName, FileMode.Open, FileAccess.Read);
                toolStripStatusLabel1.Text = "Loading datafile...";
                ProgressBar.Visible = true;
                operationButton.Visible = true;
                WorkingThread = new Thread(new ParameterizedThreadStart(LoadData));
                WorkingThread.Start(file);
            }
        }

        /// <summary>
        /// Starts a new thread responsible for generating rules.
        /// </summary>
        public void StartGenerationThread()
        {
            if (this.InvokeRequired)
            {
                ThreadCallback t = new ThreadCallback(StartGenerationThread);
                this.Invoke(t, new object[] { });
            }
            else
            {
                ProgressBar.Visible = true;
                operationButton.Visible = true;
                toolStripStatusLabel1.Text = "Generating rules...";
                WorkingThread = new Thread(new ThreadStart(RuleGenerationThread));
                WorkingThread.Start();
            }
        }

        private void shoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DataGrid d = new DataGrid();
            d.LoadData(database);
            d.Show();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void createChartToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateChart c = new CreateChart(RuleStructure, ConclusionList,
                MinSup, MinConf, MinS, MinF, MaxAnti);
            c.Show();
        }

        /// <summary>
        /// Displays rules given in the <c>ruleList</c> parameter.
        /// </summary>
        /// <param name="ruleList">List of rules to be displayed</param>
        public void DisplayRules(SortableBindingList<Rule> ruleList)
        {
            try
            {
                if (this.DGV.InvokeRequired)
                {
                    DisplayRulesCallback d = new DisplayRulesCallback(DisplayRules);
                    this.Invoke(d, new object[] { ruleList });
                }
                else
                {
                    ProgressBar.Visible = false;
                    operationButton.Visible = false;
                    createChartToolStripMenuItem.Enabled = true;
                    createMultichartToolStripMenuItem.Enabled = true;
                    saveRuleFileToolStripMenuItem.Enabled = true;
                    DGV.DataSource = ruleList;
                    toolStripRuleCount.Text = ruleList.Count.ToString();
                    DGV.AutoResizeColumns();
                }
            }
            catch
            {
                MessageBox.Show("Internal rule display error.", "Rule display error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Error();
            }
        }

        private void minimalSupportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MinSupp m = new MinSupp(this, database.Template);
            m.Show();
            m.textBox1.Focus();
        }

        internal void SetMinSup(double minSup)
        {
            MinSup = minSup;
        }

        private void SaveRules(object f)
        {
            try
            {
                FileStream file = (FileStream)f;
                string extension = file.Name.Substring(file.Name.Length - 3);
                if (extension == "rff")
                    FileProcessor.SaveRules(file, RuleStructure, ConclusionList.Count,
                                            MinSup, MinConf, MinS, MinF, MaxAnti);
                else if (extension == "csv") 
                {
                    FileProcessor.ExportToCSV(DGV,file);
                }
                else
                    FileProcessor.ExportToExcel(DGV,file);
                Saved();
            }
            catch
            {
                MessageBox.Show("File error! Try again with a different file name.!", "File error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Error();
            }
        }

        /// <summary>
        /// Updates status strip elements when rules are saved.
        /// </summary>
        public void Saved()
        {
            if (this.DGV.InvokeRequired)
            {
                ThreadCallback d = new ThreadCallback(Saved);
                this.Invoke(d, new object[] { });
            }
            else
            {
                toolStripStatusLabel1.Text = "Rules saved!";
                ProgressBar.Visible = false;
                operationButton.Visible = false;
            }
        }

        private void saveRuleFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveRuleFileDialog.Title = "Save as...";
            saveRuleFileDialog.Filter = "Rule File (*.rff)|*.rff|Excel 2003 (*.xls)|*.xls|"+
                                        "Excel XML File (*.xml)|*.xml|Comma Seperated Values (*.csv)|*.csv";
            saveRuleFileDialog.FilterIndex = 1;
            saveRuleFileDialog.FileName = "";
            saveRuleFileDialog.OverwritePrompt = true;
            if (saveRuleFileDialog.ShowDialog() != DialogResult.Cancel)
            {
                FileStream file = new FileStream(saveRuleFileDialog.FileName, FileMode.Create, FileAccess.Write);
                ProgressBar.Visible = true;
                toolStripStatusLabel1.Text = "Saving rule file...";
                operationButton.Visible = true;
                WorkingThread = new Thread(new ParameterizedThreadStart(SaveRules));
                WorkingThread.Start(file);
            }
        }

        private void LoadRules(object file)
        {
            ConclusionList = new List<Set>();
            ruleList = new SortableBindingList<Rule>();
            try
            {
                RuleStructure = FileProcessor.OpenRuleFile((FileStream)file, ConclusionList, ruleList,
                                out MinSup, out MinConf, out MinS, out MinF, out MaxAnti);
                toolStripStatusLabel1.Text = "Rules loaded";
                DisplayRules(ruleList);
            }
            catch 
            { 
                MessageBox.Show("Wrong file format!", "File format error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Error();
            }
        }

        private void Error()
        {
            if (this.InvokeRequired)
            {
                ThreadCallback d = new ThreadCallback(Error);
                this.Invoke(d, new object[] { });
            }
            else
            {
                operationButton.Visible = false;
                toolStripStatusLabel1.Text = "Operation aborted!";
                ProgressBar.Visible = false;
                WorkingThread.Abort();
            }
        }

        private void openRuleFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openRuleFileDialog.Title = "Choose file...";
            openRuleFileDialog.Filter = "Rule File (*.rff)|*.rff";
            //openRuleFileDialog.FilterIndex = 1;
            openRuleFileDialog.FileName = "";
            if (openRuleFileDialog.ShowDialog() != DialogResult.Cancel)
            {
                saveRuleFileToolStripMenuItem.Enabled = false;

                FileStream file = new FileStream(openRuleFileDialog.FileName, FileMode.Open, FileAccess.Read);
                toolStripStatusLabel1.Text = "Loading rule file...";
                operationButton.Visible = true;
                ProgressBar.Visible = true;
                WorkingThread = new Thread(new ParameterizedThreadStart(LoadRules));
                WorkingThread.Start(file);
            }
        }

        private void createMultichartToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateMultiChart mc = new CreateMultiChart(RuleStructure, ConclusionList, 
                                    MinSup, MinConf, MinS, MinF, MaxAnti);
            mc.Show();
        }

        private void abortOperationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            operationButton.Visible = false;
            toolStripStatusLabel1.Text = "Operation aborted!";
            ProgressBar.Visible = false;
            WorkingThread.Abort();
        }

        private void pauseOperationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (WorkingThread.ThreadState == ThreadState.Running)
            {
                toolStripStatusLabel1.Text += " (paused)";
                ProgressBar.Visible = false;
                WorkingThread.Suspend();
            }
        }

        private void resumeOperationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (WorkingThread.ThreadState == ThreadState.Suspended)
            {
                string text = toolStripStatusLabel1.Text;
                toolStripStatusLabel1.Text = text.Substring(0, text.Length - 9);
                ProgressBar.Visible = true;
                WorkingThread.Resume();
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetDataObject(DGV.GetClipboardContent());
        }

        private void helpTopicsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                Help.ShowHelp(this, CNetHelpProvider.HelpNamespace);
            }
            catch
            {
                MessageBox.Show("Help data not attached!", "Help error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Error();
            }
        }

        internal void SetMinConf(double m)
        {
            MinConf = m;
        }

        internal void SetMinF(double m)
        {
            MinF = m;
        }

        internal void SetMinS(double m)
        {
            MinS = m;
        }

        internal void SetMaxAnti(double m)
        {
            MaxAnti = m;
        }

        internal void SetDecClass(string dc)
        {
            decClass = dc;
        }

        private void aprioriToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!aprioriToolStripMenuItem.Checked)
            {
                aprioriToolStripMenuItem.Checked = true;
                fPGrowthToolStripMenuItem.Checked = false;
            }
        }

        private void fPGrowthToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!fPGrowthToolStripMenuItem.Checked)
            {
                aprioriToolStripMenuItem.Checked = false;
                fPGrowthToolStripMenuItem.Checked = true;
            }
        }
    }
}