using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Association_mining
{
    /// <summary>
    /// Provides an interface to provide minimal 
    /// measure values required later to generate rules.
    /// </summary>
    public partial class MinSupp : Form
    {
        private frmMain MainForm;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="parent">Parent form</param>
        public MinSupp(frmMain parent, RecordHolder template)
        {
            MainForm = parent;
            InitializeComponent();
            cboxDecision.SelectedIndex = 0;
            foreach (String attributeName in template.DistinctTypes)
                cboxDecision.Items.Add(attributeName);
        }

        /// <summary>
        /// Destroys the form.
        /// </summary>
        protected void Cancel_Click(object sender, EventArgs e)
        {
            Dispose();
        }
        
        /// <summary>
        /// Starts the rule generation thread if the values
        /// provided by the user are correct.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void OK_Click(object sender, EventArgs e)
        {
            double minSup, minConf, minS, minF, maxAnti;
            try
            {
                if (Double.TryParse(textBox1.Text, out minSup) && minSup > 0 && minSup <= 1)
                {
                    if ((!Double.TryParse(tboxConf.Text, out minConf) && tboxConf.Text.Length>0) || minConf < 0 || minConf> 1)               
                        throw new Exception("Wrong confidence value! Type in anumber between 0 and 1 or leave the field empty.");
                    if ((!Double.TryParse(tboxS.Text, out minS) && tboxS.Text.Length > 0 ) || minS < -1 || minS > 1)
                        throw new Exception("Wrong confirmation-s value! Type in anumber between -1 and 1 or leave the field empty.");
                    if ((!Double.TryParse(tboxF.Text, out minF) && tboxF.Text.Length > 0) || minF < -1 || minF > 1)
                        throw new Exception("Wrong confirmation-f value! Type in anumber between -1 and 1 or leave the field empty.");
                    if (( !Double.TryParse(tboxAnti.Text, out maxAnti) &&tboxAnti.Text.Length>0) || maxAnti < 0 || maxAnti > 1)
                        throw new Exception("Wrong anti-support value! Type in anumber between 0 and 1 or leave the field empty.");
                        
                    if (tboxConf.Text.Length==0) minConf = 0;
                    if (tboxS.Text.Length == 0) minS = -1;
                    if (tboxF.Text.Length == 0) minF = -1;
                    if (tboxAnti.Text.Length == 0) maxAnti = 1;


                    MainForm.SetMinSup(minSup);
                    MainForm.SetMinConf(minConf);
                    MainForm.SetMinS(minS);
                    MainForm.SetMinF(minF);
                    MainForm.SetMaxAnti(maxAnti);

                    if (cboxDecision.SelectedIndex > 0)
                        MainForm.SetDecClass(cboxDecision.SelectedItem.ToString());
                    else
                        MainForm.SetDecClass("");

                    Hide();
                    MainForm.StartGenerationThread();
                    Dispose();
                }
                else
                    throw new Exception("Minmal support must be a floating point value greater than 0,0 and less or equal 1,0!");
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message, "Invalid value", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }              
        }
    }
}